//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
`include "sd_parms.v"

module sopc (
  clk_pos_i, clk_neg_i, btn_i, sin_i, sw_i, sysace_clk_i,
  sout_o, led_o,
  // System ACE MPU port interface
  mpbrdy_i, mpirq_i,
  mpce_n_o, mpwe_n_o, mpoe_n_o, mpa_o, mpd_io
  //
  );
  
  input clk_pos_i;                // Positive differential clock input
  input clk_neg_i;                // Negative differential clock input
  input [3:0] btn_i;              // Button inputs
  input sin_i;                    // UART input
  input [3:0] sw_i;               // Input from DIP switches
  input sysace_clk_i;             // Input clock used for SYSACE controller
  output sout_o;                  // UART output
  output [3:0] led_o;             // LED outputs
  // System ACE MPU port interface
  input mpbrdy_i;                 // Buffer ready
  input mpirq_i;                  // Interrupt request
  output mpce_n_o;                // Chip enable
  output mpwe_n_o;                // Write enable
  output mpoe_n_o;                // Output enable
  output [6:0] mpa_o;             // Address lines
  inout [7:0] mpd_io;             // Data lines
  
  assign led_o[3:0] = (sw_i == 4'h0) ? {mpce_n_o,mpwe_n_o,mpoe_n_o,mpbrdy_i} :
                      (sw_i == 4'h1) ? sd_test_w : 
                      (sw_i == 4'h2) ? test1_w :
                      (sw_i == 4'h3) ? test2_w[3:0] :
                      (sw_i == 4'h4) ? test2_w[7:4] :
                      (sw_i == 4'h5) ? test2_w[11:8] :
                      (sw_i == 4'h6) ? test2_w[15:12] :
                      4'b1010;
  
  // Internal wires and registers
  reg uart_cs_r = 1'b1;
  reg [2:0] uart_adr_r = 3'b000;
  wire [7:0] uart_data_w;
  wire uart_rd_w;
  wire uart_wr_w;
  wire [7:0] uart_datar_w;
  wire uart_txrdy_w;
  wire uart_rxrdy_w;
  reg [3:0] cmd_r;
  
  wire [31:0] wb_adri_w;                // I/O direction relatively to sdcntr_top module
  wire [31:0] wb_datai_w;
  wire wb_wei_w;
  wire [3:0] wb_seli_w;
  wire wb_stbi_w;
  wire wb_acki_w;
  wire wb_cyci_w;
  wire wb_tagni_w;
  wire [31:0] wb_adro_w;
  wire [31:0] wb_datao_w;
  wire wb_weo_w;
  wire [3:0] wb_selo_w;
  wire wb_stbo_w;
  wire wb_acko_w;
  wire wb_cyco_w;
  wire wb_tagno_w;
  wire clk_gp; 
  reg init_done;
  reg [11:0] init_counter = 12'h0;
  wire rst_w = (init_done == 1'b0) ? 1'b0 : ~btn_i[0];
  wire [3:0] sd_test_w;		   
  wire [3:0] test1_w;		   
  wire [15:0] test2_w;
  // Modules instantiation
  //
  						 
  assign clk_gp = sysace_clk_i;
  
  clk_wiz_v1_7_0 instance_name
   (// Clock in ports
    .CLK_IN1_P          (clk_pos_i),    // IN
    .CLK_IN1_N          (clk_neg_i),    // IN
    // Clock out ports
    .CLK_OUT1           ());    // OUT  30 MHz
    
  sdcard_cntr sdcntr_top  (
     .clk_i           (clk_gp),
     .rst_i           (rst_w),
     .mpbrdy_i        (mpbrdy_i), 
     .mpirq_i         (mpirq_i),
     .mpce_n_o        (mpce_n_o), 
     .mpwe_n_o        (mpwe_n_o), 
     .mpoe_n_o        (mpoe_n_o), 
     .mpa_o           (mpa_o),
     .mpd_io          (mpd_io),
     .wb_adr_i        (wb_adri_w), 
     .wb_data_i       (wb_datai_w), 
     .wb_we_i         (wb_wei_w), 
     .wb_sel_i        (wb_seli_w), 
     .wb_stb_i        (wb_stbi_w), 
     .wb_ack_i        (wb_acki_w),
     .wb_cyc_i        (wb_cyci_w), 
     .wb_tagn_i       (wb_tagni_w),
     .wb_adr_o        (wb_adro_w), 
     .wb_data_o       (wb_datao_w), 
     .wb_we_o         (wb_weo_w), 
     .wb_sel_o        (wb_selo_w), 
     .wb_stb_o        (wb_stbo_w), 
     .wb_ack_o        (wb_acko_w),
     .wb_cyc_o        (wb_cyco_w), 
     .wb_tagn_o       (wb_tagno_w),
     .led_o           (sd_test_w),
     .test1_o         (test1_w),
     .test2_o         (test2_w)
  );

    uart_top uart_top_sopc (
  	.clk_i                      (clk_gp),
  	.rst_i                      (rst_w),
  	.sin_i                      (sin_i),
  	.ctsn_i                     (),
  	.sout_o                     (sout_o),
  	.rtsn_o                     (),
  	.data_i                     (uart_data_w),
  	.rd_i                       (uart_rd_w),
  	.wr_i                       (uart_wr_w),
  	.adr_i                      (uart_adr_r),
  	.cs_i                       (uart_cs_r),
  	.data_o                     (uart_datar_w),
  	.irq_o                      (),
  	.txrdy_o                    (uart_txrdy_w),
  	.rxrdy_o                    (uart_rxrdy_w)  	 	
    ); 	
    
  uart_cntr uart_cntr_sopc (
  	.clk_i                      (clk_gp), 
  	.rst_i                      (rst_w), 
  	.data_i                     (), 
  	.uart_data_i                (uart_datar_w), 		
  	.uart_txrdy_i               (uart_txrdy_w),
  	.uart_rxrdy_i               (uart_rxrdy_w),
  	.uart_we_o                  (uart_wr_w),
  	.uart_rd_o                  (uart_rd_w),
  	.data_o                     (uart_data_w),
  	.wb_data_i                  (wb_datao_w),
  	.wb_adr_i                   (wb_adro_w),
  	.wb_we_i                    (wb_weo_w),
  	.wb_sel_i                   (wb_selo_w),
  	.wb_stb_i                   (wb_stbo_w),
  	.wb_ack_i                   (wb_acko_w),
  	.wb_cyc_i                   (wb_cyco_w),
  	.wb_data_o                  (wb_datai_w),
  	.wb_adr_o                   (wb_adri_w),
  	.wb_we_o                    (wb_wei_w),
  	.wb_sel_o                   (wb_seli_w),
  	.wb_stb_o                   (wb_stbi_w),
  	.wb_ack_o                   (wb_acki_w),
  	.wb_cyc_o                   (wb_cyci_w),
  	.uart_led_o                 ()
  );
  
  always @ (posedge clk_gp)
    begin
      if (init_counter != 12'hFFF)
        begin
          init_counter <= init_counter + 1'b1;
          init_done <= 1'b0;
        end					
      else 
        init_done <= 1'b1;
    end
endmodule